@@ -1,9 +1,10 @@
 """Functions for computing the Kernighan–Lin bipartition algorithm."""
 
-import networkx as nx
 from itertools import count
-from networkx.utils import not_implemented_for, py_random_state, BinaryHeap
+
+import networkx as nx
 from networkx.algorithms.community.community_utils import is_partition
+from networkx.utils import BinaryHeap, not_implemented_for, py_random_state
 
 __all__ = ["kernighan_lin_bisection"]
 
@@ -39,21 +40,23 @@ def _kernighan_lin_sweep(edges, side):
         yield totcost, i, (u, v)
 
 
-@py_random_state(4)
 @not_implemented_for("directed")
+@py_random_state(4)
+@nx._dispatchable(edge_attrs="weight")
 def kernighan_lin_bisection(G, partition=None, max_iter=10, weight="weight", seed=None):
     """Partition a graph into two blocks using the Kernighan–Lin
     algorithm.
 
     This algorithm partitions a network into two sets by iteratively
     swapping pairs of nodes to reduce the edge cut between the two sets.  The
-    pairs are chosen according to a modified form of Kernighan-Lin, which
+    pairs are chosen according to a modified form of Kernighan-Lin [1]_, which
     moves node individually, alternating between sides to keep the bisection
     balanced.
 
     Parameters
     ----------
-    G : graph
+    G : NetworkX graph
+        Graph must be undirected.
 
     partition : tuple
         Pair of iterables containing an initial partition. If not
@@ -61,7 +64,7 @@ def kernighan_lin_bisection(G, partition=None, max_iter=10, weight="weight", see
 
     max_iter : int
         Maximum number of times to attempt swaps to find an
-        improvemement before giving up.
+        improvement before giving up.
 
     weight : key
         Edge data key to use as weight. If None, the weights are all
@@ -100,8 +103,8 @@ def kernighan_lin_bisection(G, partition=None, max_iter=10, weight="weight", see
     else:
         try:
             A, B = partition
-        except (TypeError, ValueError) as e:
-            raise nx.NetworkXError("partition must be two sets") from e
+        except (TypeError, ValueError) as err:
+            raise nx.NetworkXError("partition must be two sets") from err
         if not is_partition(G, (A, B)):
             raise nx.NetworkXError("partition invalid")
         side = [0] * n
