@@ -1,19 +1,11 @@
-# test_mycielski.py - unit tests for the mycielski module
-#
-# Copyright 2010, 2011, 2012, 2013, 2014, 2015 NetworkX developers.
-#
-# This file is part of NetworkX.
-#
-# NetworkX is distributed under a BSD license; see LICENSE.txt for more
-# information.
-
 """Unit tests for the :mod:`networkx.generators.mycielski` module."""
 
+import pytest
+
 import networkx as nx
 
 
-class TestMycielski(object):
-
+class TestMycielski:
     def test_construction(self):
         G = nx.path_graph(2)
         M = nx.mycielskian(G)
@@ -34,3 +26,5 @@ class TestMycielski(object):
         assert nx.is_isomorphic(G, nx.cycle_graph(5))
         G = nx.mycielski_graph(4)
         assert nx.is_isomorphic(G, nx.mycielskian(nx.cycle_graph(5)))
+        with pytest.raises(nx.NetworkXError, match="must satisfy n >= 1"):
+            nx.mycielski_graph(0)
